
#include "SRAM.h"
#include <ctype.h>
#include <system.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

static unsigned int sram_stack_ptr;
static unsigned int sram_stack_start;

//writes len bytes from buffer to the sram stack, returns true if success
bool sram_push(unsigned char * buffer, size_type len){
	if (sram_stack_ptr+len < SRAM_MAX_SIZE){
		if (buffer!=NULL) sram_write(sram_stack_ptr, buffer, len);
		sram_stack_ptr+=len;
		return true;
	}else{
		return false;
	}
}

//removes len bytes from the sram stack, saving them in the buffer
void sram_pop(unsigned char * buffer, size_type len){
	sram_stack_ptr -= len;
	if (buffer!=NULL) sram_read(sram_stack_ptr,buffer,len);
	if (sram_stack_ptr < sram_stack_start) sram_stack_ptr=sram_stack_start;
}

//returns a ptr to the next free stack byte
unsigned int sram_get_stack_ptr(){
	return sram_stack_ptr;
}

void sram_set_stack_ptr(unsigned int ptr){
	sram_stack_ptr = ptr;
}

//sets the start of the stack and initializes the stack pointer to this location
void sram_set_stack_start(unsigned int start){
	sram_stack_start=start;
	sram_stack_ptr=start;
}


//to be called after sram_open, writes a byte and increments the address
void sram_write_byte(unsigned char byte){
	sram_spi_transmit(byte);
}

//to be called after sram_open, reads a byte and increments the address
unsigned char sram_read_byte(){
	return sram_spi_read();
}

//loads address into the sram chip. If read is set to true then sram_get can be used
//to get a byte from the memory, calling sram_get again will increase address automatically
void sram_open(unsigned int address, bool read){
	unsigned char TmpAddress=0;
	sram_cs=0;
	sram_write_byte(read ? 3 : 2);
	
	//write the address
	HIBYTE(TmpAddress,address);
	sram_write_byte(TmpAddress);
	LOBYTE(TmpAddress,address);
	sram_write_byte(TmpAddress);
}

//to be called after sram_open it disables the chip
//enables other functions again.
void sram_close(){
   sram_cs=1;
}

//initializes external sram
//initializes for sequential mode
//enables hold
void sram_init(){
	//set the right 3state values
	sram_tris_cs = 0;
	#ifndef sram_no_hold_pin
		sram_tris_hold = 0;
		sram_hold=1; //don't hold
	#endif
	
	sram_cs=1; //do not select the chip
	delay_us(1);
	sram_cs=0;//select chip
	sram_write_byte(1);//write to status register
	sram_write_byte(0b01000000); //enable chip for sequential mode
	sram_cs=1;
	
}

//reads the status register
unsigned char sram_readstatus(){
	unsigned char res;
	sram_cs=0;
	nop();
	sram_write_byte(0b101);
	res = sram_read_byte();
	sram_cs=1;
}


//writes len bytes stored in data to the address
void sram_write(unsigned int address,unsigned char * data,unsigned char len){
	sram_open(address,false);
	for (unsigned char i=0;i<len;i++){
		sram_write_byte(data[i]);
	}
	sram_close();
}

void sram_write(unsigned int address,unsigned char * data,unsigned int len){
	sram_open(address,false);
	for (unsigned int i=0;i<len;i++){
		sram_write_byte(data[i]);
	}
	sram_close();
}

//write string to sram
inline void sram_write_string(unsigned int address, unsigned char * data){
	sram_write(address,data,strlen(data));
}

//reads len bytes from address and puts in data
void sram_read(unsigned int address, unsigned char * data, unsigned char len){
	sram_open(address,true);
	//input all bytes
	for (unsigned char i=0;i<len;i++){
		data[i]=sram_read_byte();
	}
	sram_close();
}

void sram_read(unsigned int address, unsigned char * data, unsigned int len){
	sram_open(address,true);
	//input all bytes
	for (unsigned int i=0;i<len;i++){
		data[i]=sram_read_byte();
	}
	sram_close();
}

//reads a null terminated string from address
void sram_read_string(unsigned int address, const unsigned char * data){
	unsigned char byte=0;
	sram_open(address,true);
	byte = sram_read_byte();
	while (byte !=0){
		*data=byte;
		byte = sram_read_byte();
		data++;
	}
	*data=0;
	sram_close();
}

//reads an unsigned interger from start address
unsigned int sram_read_int(unsigned int start){
	unsigned char byte=0;
	unsigned int res=0;
	sram_open(start,true);
	byte = sram_read_byte();
	while ('0' <= byte && byte <= '9'){
		res = res * 10 + byte;
		byte = sram_read_byte();
	}
	sram_close();
	return res;
}


//searches for a sequence in the sram
//sequence max lenght of 255 characters
int sram_find_array(unsigned int Start, unsigned int Stop, unsigned char * Sequence, size_type SequenceCnt, bool CaseSensetive){
	unsigned char Data;
	unsigned int LoopEnd = Stop-SequenceCnt+1;
	unsigned char j=0;
	unsigned char Seq;
	sram_open(Start,true);
	for(unsigned int i=Start;i<=LoopEnd;i++){
		if (CaseSensetive){
			Data = sram_read_byte(); //read in a byte
			Seq = Sequence[j];
		}else{
			Data = toupper(sram_read_byte()); //read in a byte
			Seq = toupper(Sequence[j]);
		}
		while (Data==Seq){ //keep reading bytes as long as we find it match the sequence
			//Data = sram_read_byte();
			//if(!CaseSensetive) Data=toupper(Data);
			j++;
			if (j==SequenceCnt) {
				sram_close(); 
				return i; //all bytes match, return i as the start of sequence in memory
			}
			if (CaseSensetive){
				Data = sram_read_byte(); //read in a byte
				Seq = Sequence[j];
			}else{
				Data = toupper(sram_read_byte()); //read in a byte
				Seq = toupper(Sequence[j]);
			}
		}
		if (j!=0){
			sram_close(); 
			j=0;
			sram_open(i+1,true); //reset the memory to the next location
		}
	}
	sram_close();
	return -1;
}

int sram_find_string(unsigned int start, unsigned int stop, unsigned char * find, bool CaseSensetive){
	return sram_find_array(start,stop,find,strlen(find),CaseSensetive);
}

//copies data from src to dst (non overlapping!)
void sram_copy(unsigned int src, unsigned int dst, unsigned int len){
	const unsigned char cpylen=16;
	unsigned char * data =(unsigned char *) alloc(cpylen);
	if (data!=0){
		unsigned char loops=len/cpylen;
		unsigned char left=len%cpylen;
		for (unsigned char i=0;i<loops;i++){
			sram_read(src,data,cpylen);
			sram_write(dst,data,cpylen);
			src+=cpylen;
			dst+=cpylen;
		}
		sram_read(src,data,left);
		sram_write(dst,data,left);
		free(data);
	}
}

void sram_replace_string(unsigned int start, unsigned int stop, unsigned int maxcnt, unsigned char * find, unsigned char * replace){
	/*unsigned int pos = sram_find_string(start,stop,find);
	unsigned int cnt=0;
	unsigned char rpllen = strlen(replace);
	unsigned char fndlen = strlen(find);
	
	while (pos!=-1 && cnt < maxcnt){
		if (rpllen==fndlen){
			sram_write(pos,replace,rpllen);
		}else if (rpllen < fndlen){
			for (unsigned int i=stop;i>)sram_write(i,
		}else if (rpllen > fndlen){
		
		}
		cnt++;
		pos = sram_find_string(start,stop,find);
	}*/

}

//fills space in the ram with val (could be used to clear it)
void sram_fill_val(unsigned int start, unsigned int len, unsigned char val){
	sram_open(start,false);
	for (unsigned int i=0;i<len;i++) sram_write_byte(val);
	sram_close();
}