/*********************************************************************
 *
 *           Helper Functions for Microchip TCP/IP Stack
 *
 *********************************************************************
 * FileName:		Helpers.C
 * Dependencies:	None
 * Processor:       PIC18, PIC24F, PIC24H, dsPIC30F, dsPIC33F, PIC32
 * Processor:       PIC18, PIC24F, PIC24H, dsPIC30F, dsPIC33F, PIC32
 * Compiler:        Microchip C32 v1.05 or higher
 *					Microchip C30 v3.12 or higher
 *					Microchip C18 v3.30 or higher
 *					HI-TECH PICC-18 PRO 9.63PL2 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright (C) 2002-2009 Microchip Technology Inc.  All rights
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and
 * distribute:
 * (i)  the Software when embedded on a Microchip microcontroller or
 *      digital signal controller product ("Device") which is
 *      integrated into Licensee's product; or
 * (ii) ONLY the Software driver source files ENC28J60.c, ENC28J60.h,
 *		ENCX24J600.c and ENCX24J600.h ported to a non-Microchip device
 *		used in conjunction with a Microchip ethernet controller for
 *		the sole purpose of interfacing with the ethernet controller.
 *
 * You should refer to the license agreement accompanying this
 * Software for additional information regarding your rights and
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date    Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Nilesh Rajbharti     5/17/01 Original        (Rev 1.0)
 * Nilesh Rajbharti     2/9/02  Cleanup
 * Nilesh Rajbharti     6/25/02 Rewritten CalcIPChecksum() to avoid
 *                              multi-byte shift operation.
 * Howard Schlunder		2/9/05  Added hexatob(), btohexa_high(), and
 *							    btohexa_low()
 * Howard Schlunder    10/10/06 Optimized swapl()
 * Elliott Wood		   11/20/07	Added leftRotateDWORD()
 ********************************************************************/

#include "Helpers.h"

/*****************************************************************************
  Function:
	DWORD GenerateRandomDWORD(void)
  Summary:
	Generates a random DWORD.
  ***************************************************************************/
DWORD GenerateRandomDWORD(void){
	DWORD_VAL dw;
	dw.w[0] = rand();
	dw.w[1] = rand();
	return dw.Val;
}

/*****************************************************************************
  Function:
	BOOL StringToIPAddress(BYTE* str, IP_ADDR* IPAddress)
  Summary:
	Converts a string to an IP address
  Description:
	This function parses a dotted-quad decimal IP address string into an 
	IP_ADDR struct.  The output result is big-endian.
  Precondition:
	None
  Parameters:
	str - Pointer to a dotted-quad IP address string
	IPAddress - Pointer to IP_ADDR in which to store the result
  Return Values:
  	TRUE - an IP address was successfully decoded
  	FALSE - no IP address could be found, or the format was incorrect
  ***************************************************************************/
BOOL StringToIPAddress(BYTE* str, IP_ADDR* IPAddress){
	DWORD_VAL dwVal;
	BYTE i, charLen, currentOctet;

	charLen = 0;
	currentOctet = 0;
	dwVal.v[0] = 0;
	dwVal.v[1] = 0;
	while((i = *str++))
	{
		if(currentOctet > 3u)
			break;

		i -= '0';
		

		// Validate the character is a numerical digit or dot, depending on location
		if(charLen == 0u){
			if(i > 9u)return FALSE;
		}else if(charLen == 3u){
			if(i != (BYTE)('.' - '0')) return FALSE;
			if((DWORD) dwVal > (DWORD)0x00020505) return FALSE;

			IPAddress->v[currentOctet++] = dwVal.v[2]*((BYTE)100) + dwVal.v[1]*((BYTE)10) + dwVal.v[0];
			charLen = 0;
			(DWORD) dwVal = 0;
			continue;
		}else{
			if(i == (BYTE)('.' - '0')){
				if((DWORD)dwVal > (DWORD)0x00020505) return FALSE;
				IPAddress->v[currentOctet++] = dwVal.v[2]*((BYTE)100) + dwVal.v[1]*((BYTE)10) + dwVal.v[0];
				charLen = 0;
				(DWORD)dwVal = 0;
				continue;
			}
			if(i > 9u) return FALSE;
		}
		charLen++;
		(DWORD)dwVal <<= 8;
		dwVal.v[0] = i;
	}
	// Make sure the very last character is a valid termination character 
	// (i.e., not more hostname, which could be legal and not an IP 
	// address as in "10.5.13.233.picsaregood.com"
	if(i != 0u && i != '/' && i != '\r' && i != '\n' && i != ' ' && i != '\t')return FALSE;

	// Verify and convert the last octet and return the result
	if((DWORD)dwVal > (DWORD)0x00020505)return FALSE;

	IPAddress->v[3] = dwVal.v[2]*((BYTE)100) + dwVal.v[1]*((BYTE)10) + dwVal.v[0];
	return true;
}


/*****************************************************************************
  Function:
	void uitoa(WORD Value, BYTE* Buffer)
  Summary:
	Converts an unsigned integer to a decimal string.
  Description:
	Converts a 16-bit unsigned integer to a null-terminated decimal string.
  Precondition:
	None
  Parameters:
	Value	- The number to be converted
	Buffer	- Pointer in which to store the converted string
  Returns:
  	None
  ***************************************************************************/
void uitoa(WORD Value, BYTE* Buffer){
	sprintf(Buffer,"%d",Value);
}

/*****************************************************************************
  Function:
	void ultoa(DWORD Value, BYTE* Buffer)
  Summary:
	Converts an unsigned integer to a decimal string.
  Description:
	Converts a 32-bit unsigned integer to a null-terminated decimal string.
  Precondition:
	None
  Parameters:
	Value	- The number to be converted
	Buffer	- Pointer in which to store the converted string
  Returns:
  	None
  ***************************************************************************/
void ultoa(DWORD Value, BYTE* Buffer){
	sprintf32(Buffer,"%d",Value);
}

/*****************************************************************************
  Function:
	BYTE hexatob(WORD_VAL AsciiChars)
  Summary:
	Converts a hex string to a single byte.
  Description:
	Converts a two-character ASCII hex string to a single packed byte.
  Precondition:
	None
  Parameters:
	AsciiChars - WORD_VAL where .v[0] is the ASCII value for the lower nibble
					and .v[1] is the ASCII value for the upper nibble.  Each
					must range from '0'-'9', 'A'-'F', or 'a'-'f'.
  Returns:
  	Resulting packed byte 0x00 - 0xFF.
  ***************************************************************************/
/*BYTE hexatob(WORD_VAL AsciiChars)
{
	// Convert lowercase to uppercase
	if(AsciiChars.v[1] > 'F')
		AsciiChars.v[1] -= 'a'-'A';
	if(AsciiChars.v[0] > 'F')
		AsciiChars.v[0] -= 'a'-'A';

	// Convert 0-9, A-F to 0x0-0xF
	if(AsciiChars.v[1] > '9')
		AsciiChars.v[1] -= 'A' - 10;
	else
		AsciiChars.v[1] -= '0';

	if(AsciiChars.v[0] > '9')
		AsciiChars.v[0] -= 'A' - 10;
	else
		AsciiChars.v[0] -= '0';

	// Concatenate
	return (AsciiChars.v[1]<<4) |  AsciiChars.v[0];
}*/

/*****************************************************************************
  Function:
	BYTE btohexa_high(BYTE b)

  Summary:
	Converts the upper nibble of a binary value to a hexadecimal ASCII byte.

  Description:
	Converts the upper nibble of a binary value to a hexadecimal ASCII byte.
	For example, btohexa_high(0xAE) will return 'A'.

  Precondition:
	None

  Parameters:
	b - the byte to convert

  Returns:
  	The upper hexadecimal ASCII byte '0'-'9' or 'A'-'F'.
  ***************************************************************************/
/*BYTE btohexa_high(BYTE b)
{
	b >>= 4;
	return (b>0x9u) ? b+'A'-10:b+'0';
}*/

/*****************************************************************************
  Function:
	BYTE btohexa_high(BYTE b)

  Summary:
	Converts the lower nibble of a binary value to a hexadecimal ASCII byte.

  Description:
	Converts the lower nibble of a binary value to a hexadecimal ASCII byte.
	For example, btohexa_high(0xAE) will return 'E'.

  Precondition:
	None

  Parameters:
	b - the byte to convert

  Returns:
  	The lower hexadecimal ASCII byte '0'-'9' or 'A'-'F'.
  ***************************************************************************/
/*BYTE btohexa_low(BYTE b)
{
	b &= 0x0F;
	return (b>9u) ? b+'A'-10:b+'0';
}*/

/*****************************************************************************
  Function:
	signed char stricmppgm2ram(BYTE* a, ROM BYTE* b)

  Summary:
	Case-insensitive comparison of a string in RAM to a string in ROM.

  Description:
	Performs a case-insensitive comparison of a string in RAM to a string
	in ROM.  This function performs identically to strcmppgm2ram, except that
	the comparison is not case-sensitive.

  Precondition:
	None

  Parameters:
	a - Pinter to tring in RAM
	b - Pointer to string in ROM

  Return Values:
  	\-1 - a < b
  	0	- a = b
  	1	- a > b
  ***************************************************************************/
/*signed char stricmppgm2ram(BYTE* a, ROM BYTE* b)
{
	BYTE cA, cB;
	
	// Load first two characters
	cA = *a;
	cB = *b;
	
	// Loop until one string terminates
	while(cA != '\0' && cB != '\0')
	{
		// Shift case if necessary
		if(cA >= 'a' && cA <= 'z')
			cA -= 'a' - 'A';
		if(cB >= 'a' && cB <= 'z')
			cB -= 'a' - 'A';
			
		// Compare
		if(cA > cB)
			return 1;
		if(cA < cB)
			return -1;
		
		// Characters matched, so continue
		a++;
		b++;
		cA = *a;
		cB = *b;
	}
	
	// See if one string terminated first
	if(cA > cB)
		return 1;
	if(cA < cB)
		return -1;
		
	// Strings match
	return 0;
}*/

//swap the nibbles of a byte
unsigned char swapb(unsigned char val){
	return (val & 0xF0)>>4 | (val & 0x0F)<<4;
}

/*****************************************************************************
  Function:
	WORD swaps(WORD v)
  Description:
	Swaps the endian-ness of a WORD.
  Precondition:
	None
  Parameters:
	v - the WORD to swap
  Returns:
	The swapped version of v.
  ***************************************************************************/
unsigned int swaps(unsigned int val){
	WORD_VAL res;
	res.v[0] = ((WORD_VAL)val).v[1];
	res.v[1] = ((WORD_VAL)val).v[0];
	return (unsigned int) res;
}

/*****************************************************************************
  Function:
	DWORD swapl(DWORD v)
  Description:
	Swaps the endian-ness of a DWORD.
  Precondition:
	None
  Parameters:
	v - the DWORD to swap
  Returns:
	The swapped version of v.
  ***************************************************************************/
DWORD swapl(DWORD v){
	DWORD_VAL res;
	res.v[0] =((DWORD_VAL)v).v[3];
	res.v[3] =((DWORD_VAL)v).v[0];
	res.v[1] =((DWORD_VAL)v).v[2];
	res.v[2] =((DWORD_VAL)v).v[1];
	// Swap bytes 0 and 3
	/*((DWORD_VAL*)&v)->v[0] ^= ((DWORD_VAL*)&v)->v[3];
	((DWORD_VAL*)&v)->v[3] ^= ((DWORD_VAL*)&v)->v[0];
	((DWORD_VAL*)&v)->v[0] ^= ((DWORD_VAL*)&v)->v[3];*/

	// Swap bytes 1 and 2
	/*((DWORD_VAL*)&v)->v[1] ^= ((DWORD_VAL*)&v)->v[2];
	((DWORD_VAL*)&v)->v[2] ^= ((DWORD_VAL*)&v)->v[1];
	((DWORD_VAL*)&v)->v[1] ^= ((DWORD_VAL*)&v)->v[2];*/
	return *((DWORD*)&res);
}

/*****************************************************************************
  Function:
	char* strupr(char* s)
  Summary:
	Converts a string to uppercase.
  Description:
	This function converts strings to uppercase on platforms that do not
	already have this function defined.  All lower-case characters are
	converted, an characters not included in 'a'-'z' are left as-is.
  Precondition:
	None
  Parameters:
	s - the null-terminated string to be converted.
  Returns:
	Pointer to the initial string.
  ***************************************************************************/
char* strupr(char* s){
	char *t;
	t = s;
	while(*t!=0){
		*t=toupper(*t);
		t++;
	}
	return s;
}

/*****************************************************************************
  Function:
	DWORD leftRotateDWORD(DWORD val, BYTE bits)
  Summary:
	Left-rotates a DWORD.
  Description:
	This function rotates the bits in a 32-bit DWORD left by a specific 
	number of bits.
  Precondition:
	None
  Parameters:
	val		- the DWORD to be rotated
	bits	- the number of bits by which to shift
  Returns:
	Rotated DWORD value.
  Remarks:
  ***************************************************************************/
/*inline DWORD leftRotateDWORD(DWORD val, BYTE bits){
	
	return val << bits;	 is not correct!!
}*/

/*****************************************************************************
  Function:
	void FormatNetBIOSName(BYTE Name[])
  Summary:
	Formats a string to a valid NetBIOS name.
  Description:
	This function formats a string to a valid NetBIOS name.  Names will be
	exactly 16 characters, as defined by the NetBIOS spec.  The 16th 
	character will be a 0x00 byte, while the other 15 will be the 
	provided string, padded with spaces as necessary.
  Precondition:
	None
  Parameters:
	Name - the string to format as a NetBIOS name.  This parameter must have
	  at least 16 bytes allocated.
  Returns:
	None
  ***************************************************************************/
void FormatNetBIOSName(BYTE Name[]){
	BYTE i;
	Name[15] = '\0';
	strupr((char*)Name);
	i = 0;
	while(i < 15){
		if(Name[i] == '\0'){
			while(i < 15){
				Name[i++] = ' ';
			}
			break;
		}
		i++;
	}
}

/*****************************************************************************
  Function:
	char * strnchr(const char *searchString, size_t count, char c)
  Summary:
	Searches a string up to a specified number of characters for a specific 
	character.
  Description:
	Searches a string up to a specified number of characters for a specific 
	character.  The string is searched forward and the first occurance 
	location is returned.  If the search character is not present in the 
	string, or if the maximum character count is reached first, then a NULL 
	pointer is returned.
  Precondition:
	None
  Parameters:
	searchString - Pointer to a null terminated string to search.  If count is 
		less than the string size, then the string need not be null terminated.
	count - Maximum number of characters to search before aborting.
	c - Character to search for
  Returns:
	Pointer to the first occurance of the character c in the string 
	searchString.  If the character is not found or the maximum count is 
	reached, a NULL pointer is returned.
  ***************************************************************************/
unsigned char * strnchr(const unsigned char *searchString, unsigned int count, char c){
	unsigned char c2;
	while(count--){
		c2  = *searchString++;
		if(c2 == 0u)
			return NULL;
		if(c2 == c)
			return (unsigned char*)--searchString;
	}
	return NULL;
}

unsigned int createshort(unsigned char low, unsigned char high){
	WORD_VAL res;
	res.v[0]=low;
	res.v[1]=high;
	return res.Val;
	/*unsigned int res;
	MAKESHORT(res,low,high);
	return res;*/
}

/*void memcpy(unsigned char * dst, unsigned char * src, unsigned int len){
	for (unsigned int i=0;i<len;i++) dst[i]=src[i];
}*/
