
#include "stacktask.h"

// Stack FSM states.
typedef enum {
    SM_STACK_IDLE,
    SM_STACK_MAC,
    SM_STACK_IP,
    SM_STACK_ARP,
    SM_STACK_TCP,
    SM_STACK_UDP
} SM_STACK;

static SM_STACK smStack;

NODE_INFO remoteNode;



/*********************************************************************
 * Function:        void StackInit(void)
 * Output:          Stack and its componets are initialized
 * Side Effects:    None
 * Note:            This function must be called before any of the
 *                  stack or its component routines are used.
 ********************************************************************/
void StackInit(void){
    smStack = SM_STACK_IDLE;
    // If DHCP or IP Gleaning is enabled,
    // * startup in Config Mode. /    
	#ifdef STACK_USE_IP_GLEANING
		network_set_configmode(true);
	#else
		#ifdef STACK_USE_DHCP_CLIENT
			network_set_configmode(true);
		#endif
	#endif

    MACInit();

    ARPInit();

	#ifdef STACK_USE_UDP
		UDPInit();
	#endif
	
	#ifdef STACK_USE_NBNS
		NBNSInit();
	#endif
	
	stack_sram_init(); //allow the user to set the sram options first
	
	#ifdef STACK_USE_TCP
		TCPInit();
	#endif

	#ifdef STACK_USE_BERKELEY_API
		BerkeleySocketInit();
	#endif

	#ifdef STACK_USE_DHCP_CLIENT
		DHCPInit(0);
		if(!network_dhcp_enabled()) DHCPDisable(0);
	#endif

	#ifdef STACK_USE_AUTOIP
		AutoIPInit(0);
	#endif

	#ifdef STACK_USE_DYNAMICDNS_CLIENT
		DDNSInit();
	#endif
}


/*********************************************************************
 * Function:        void StackTask(void)
 * PreCondition:    StackInit() is already called.
 * Output:          Stack FSM is executed.
 * Note:            This FSM checks for new incoming packets,
 *                  and routes it to appropriate stack components.
 *                  It also performs timed operations.
 *
 *                  This function must be called periodically to
 *                  ensure timely responses.
 ********************************************************************/
void StackTask(void){
    WORD dataCount;
    IP_ADDR tempLocalIP;
	BYTE cFrameType;
	BYTE cIPFrameType;

	#ifdef STACK_USE_DHCP_CLIENT
		// Normally, an application would not include  DHCP module
		// if it is not enabled. But in case some one wants to disable
		// DHCP module at run-time, remember to not clear our IP
		// address if link is removed.
		if(network_dhcp_enabled()){
			/*static BOOL bLastLinkState = FALSE; //if the link goes down we will reset everything, not only the DHCP server
			BOOL bCurrentLinkState;
			
			bCurrentLinkState = MACIsLinked();
			if(bCurrentLinkState != bLastLinkState){
				bLastLinkState = bCurrentLinkState;
				if(!bCurrentLinkState){
					AppConfig.MyIPAddr.Val = AppConfig.DefaultIPAddr.Val;
					AppConfig.MyMask.Val = AppConfig.DefaultMask.Val;
					AppConfig.Flags.bInConfigMode = TRUE;
					DHCPInit(0);
				}
			}*/
		
			// DHCP must be called all the time even after IP configuration is
			// discovered.
			// DHCP has to account lease expiration time and renew the configuration
			// time.
			DHCPTask();
			
			if(DHCPIsBound(0))  network_set_configmode(FALSE);
		}
	#endif

    #ifdef STACK_USE_AUTO_IP
		AutoIPTasks();
    #endif

	#ifdef STACK_USE_TCP
		//pushcs(0x04);
		TCPTick();// Perform all TCP time related tasks (retransmit, send acknowledge, close connection, etc)
		//popcs();
	#endif

	#ifdef STACK_USE_UDP
		//pushcs(0x05);
		UDPTask();
		//popcs();
	#endif

	// Process as many incomming packets as we can
	while(1){
		//if using the random module, generate entropy
		//#ifdef STACK_USE_RANDOM
		//	RandomAdd(remoteNode.MACAddr.v[5]);
		//#endif/

		// We are about to fetch a new packet, make sure that the 
		// UDP module knows that any old RX data it has laying 
		// around will now be gone.
		#ifdef STACK_USE_UDP
			UDPDiscard();
		#endif

		// Fetch a packet (throws old one away, if not thrown away yet)
		if(!MACGetHeader(&remoteNode.MACAddr, &cFrameType)) break;

		// Dispatch the packet to the appropriate handler
		switch(cFrameType){
			case MAC_ARP:
				ARPProcess();
				break;
			case MAC_IP:
				cIPFrameType=0;
				if(IPGetHeader(&tempLocalIP, &remoteNode, &cIPFrameType, &dataCount)){
					#ifdef STACK_USE_ICMP
						if(cIPFrameType == IP_PROT_ICMP){
							#ifdef STACK_USE_IP_GLEANING
								if(network_in_configmode() && network_dhcp_enabled()){
									// According to "IP Gleaning" procedure,
									// when we receive an ICMP packet with a valid
									// IP address while we are still in configuration
									// mode, accept that address as ours and conclude
									// configuration mode.
									if((DWORD)tempLocalIP != 0xffffffff){
										network_set_configmode(FALSE);
										network_set_ip (tempLocalIP);
									}
								}
							#endif
		
							// Process this ICMP packet if it the destination IP address matches our address or one of the broadcast IP addressees
							if(((DWORD)tempLocalIP == (DWORD)network_get_ip()) || ((DWORD)network_get_ip() == (DWORD)0xFFFFFFFF) ||
								((DWORD)tempLocalIP == (((DWORD)network_get_ip() & (DWORD)network_get_subnet()) | ~(DWORD)network_get_subnet()))){
								ICMPProcess(&remoteNode, dataCount);
							}
							break;
						}
					#endif
					
					#ifdef STACK_USE_TCP
						if(cIPFrameType == IP_PROT_TCP){
							//serial_send_string("TCPrx");
							TCPProcess(&remoteNode, &tempLocalIP, dataCount);
							break;
						}
					#endif
					
					#ifdef STACK_USE_UDP
						if(cIPFrameType == IP_PROT_UDP){
							//serial_send_string("UDPrx\r\n");
							// Stop processing packets if we came upon a UDP frame with application data in it
							if(UDPProcess(&remoteNode, &tempLocalIP, dataCount)) return;
						}
					#endif
				}
				break;
		}
	}
}

/*********************************************************************
 * Function:        void StackApplications(void)
 * PreCondition:    StackInit() is already called.
 * Input:           None
 * Output:          Calls all loaded application modules.
 * Side Effects:    None
 * Note:            This function must be called periodically to
 *                  ensure timely responses.
 ********************************************************************/
 
void StackApplications(void){
	
	#ifdef STACK_USE_NBNS
		NBNSTask();
	#endif
	
	#ifdef STACK_USE_DYNAMICDNS_CLIENT
		DDNSTask();
	#endif
	
	#ifdef STACK_USE_SMTP_CLIENT
		SMTPTask();
	#endif
	
	custom_network_task();
	
}
